@@ -11,7 +11,8 @@ public:
   // constructor
   ung_amcmc(const unsigned int n_iter, const unsigned int n_burnin, const unsigned int n_thin, 
     const unsigned int n, const unsigned int m, const double target_acceptance, 
-    const double gamma, const arma::mat& S, const bool store_states = true);
+    const double gamma, const arma::mat& S, const unsigned int output_type = 1, 
+    const bool store_modes = true);
   
   void expand();
   
@@ -20,7 +21,6 @@ public:
   void approx_mcmc(T model, const bool end_ram, const bool local_approx, 
     const arma::vec& initial_mode, const unsigned int max_iter, const double conv_tol);
   
-  
   template <class T>
   void is_correction_psi(T model, const unsigned int nsim_states, 
     const unsigned int is_type, const unsigned int n_threads);
@@ -47,6 +47,7 @@ private:
   arma::mat scales_storage;
   arma::vec approx_loglik_storage;
   arma::vec prior_storage;
+  const bool store_modes;
 };
 
 
