@@ -10,10 +10,10 @@ public:
   ung_bsm(const Rcpp::List& model, const unsigned int seed);
 
   // update model given the parameters theta
-  void set_theta(const arma::vec& theta);
-  // extract theta from the model
-  arma::vec get_theta() const;
-
+  void update_model(const arma::vec& new_theta);
+  double log_prior_pdf(const arma::vec& x) const;
+  double log_proposal_ratio(const arma::vec& new_theta, const arma::vec& old_theta) const;
+  
 private:
   const bool slope;
   const bool seasonal;
